@@ -22,6 +22,14 @@
 #' confint(gaussian_example_fit, level = 0.9)
 #'
 confint.dynamitefit <- function(object, parm, level = 0.95, ...) {
+  stopifnot_(
+    !missing(object),
+    "Argument {.arg x} is missing."
+  )
+  stopifnot_(
+    is.dynamitefit(object),
+    "Argument {.arg object} must be a {.cls dynamitefit} object."
+  )
   stopifnot_(
     checkmate::test_number(
       x = level,
